package nl.pim16aap2.armoredElytra.nbtEditor.ArmoredElytraBuilder;

import nl.pim16aap2.armoredElytra.util.ArmorTier;
import nl.pim16aap2.armoredElytra.util.EnchantmentContainer;
import org.bukkit.inventory.ItemStack;

/**
 * An interface describing the combination step of building an armored elytra
 */
public interface ItemCombinator {

    /**
     * Repairs the armored elytra provided as input.
     *
     * @param count The amount of repair items to process.
     * @return The next step of the builder process.
     */
    ItemMetadataSetter repair(int count);

    /**
     * Adds a set of enchantments to the armored elytra.
     *
     * @param enchantmentContainer The enchantments to add.
     * @return The next step of the builder process.
     */
    ItemMetadataSetter addEnchantments(EnchantmentContainer enchantmentContainer);

    /**
     * Adds a set of enchantments to the armored elytra.
     *
     * @param sourceItem The source item from which to take the enchantments.
     * @return The next step of the builder process.
     */
    ItemMetadataSetter addEnchantments(ItemStack sourceItem);

    /**
     * Combines the input elytra with another item.
     *
     * @param item      The item to combine with the input elytra. This can either be a chestplate or, if the input
     *                  elytra is an armored one, another armored elytra.
     * @param armorTier The armor tier of the input item. If this is not known, use {@link #combineWith(ItemStack)}
     *                  instead.
     * @return The next step of the builder process.
     */
    ItemMetadataSetter combineWith(ItemStack item, ArmorTier armorTier);

    /**
     * See {@link #combineWith(ItemStack, ArmorTier)}. Used when the armor tier of the input item is not known.
     *
     * @return The next step of the builder process.
     */
    ItemMetadataSetter combineWith(ItemStack item);

    /**
     * Upgrades the elytra to a specific armor tier.
     *
     * @param armorTier The new armor tier.
     * @return The next step of the builder process.
     */
    ItemMetadataSetter upgradeToTier(ArmorTier armorTier);

}
